import { type Cause, Context, Effect, type Equivalence, type ExecutionStrategy, Layer, Option, type Runtime, type Scope, Stream, type Tracer, type Types } from 'effect';
import * as Computed from './Computed.js';
export declare const TypeId: unique symbol;
export type TypeId = typeof TypeId;
export interface LazyRef<in out A, out E = never, out R = never> extends Computed.Computed<A, E, R> {
    readonly [TypeId]: LazyRef.Variance<A, E, R>;
    readonly shutdown: Effect.Effect<void, never, R>;
    readonly awaitShutdown: Effect.Effect<void, never, R>;
    readonly subscriberCount: Effect.Effect<number, never, R>;
    readonly runUpdates: <B, E2, R2>(f: (getSetDelete: GetSetDelete<A, E, R>) => Effect.Effect<B, E2, R2>) => Effect.Effect<B, E2, R | R2>;
}
export declare namespace LazyRef {
    interface Variance<in out A, out E, out R> {
        readonly _A: Types.Invariant<A>;
        readonly _E: Types.Covariant<E>;
        readonly _R: Types.Covariant<R>;
    }
    type Success<T> = [T] extends [{
        readonly [TypeId]: Variance<infer A, infer _, infer __>;
    }] ? A : never;
    type Error<T> = [T] extends [{
        readonly [TypeId]: Variance<infer _, infer E, infer __>;
    }] ? E : never;
    type Context<T> = [T] extends [{
        readonly [TypeId]: Variance<infer _, infer __, infer R>;
    }] ? R : never;
}
export interface SubscriptionRefOptions<A> {
    readonly eq?: Equivalence.Equivalence<A>;
    readonly executionStrategy?: ExecutionStrategy.ExecutionStrategy;
}
export interface GetSetDelete<A, E, R> {
    readonly get: Effect.Effect<A, E, R>;
    readonly set: (a: A) => Effect.Effect<A, never, R>;
    readonly delete: Effect.Effect<Option.Option<A>, E, R>;
    readonly version: Effect.Effect<number, never, R>;
}
export declare namespace GetSetDelete {
    type Success<T> = [T] extends [GetSetDelete<infer A, infer E, infer R>] ? A : never;
    type Error<T> = [T] extends [GetSetDelete<infer A, infer E, infer R>] ? E : never;
    type Context<T> = [T] extends [GetSetDelete<infer A, infer E, infer R>] ? R : never;
}
export declare const fromEffect: <A, E, R>(effect: Effect.Effect<A, E, R>, options?: SubscriptionRefOptions<A>) => Effect.Effect<LazyRef<A, E>, never, R | Scope.Scope>;
export declare const fromStream: <A, E, R>(stream: Stream.Stream<A, E, R>, options?: SubscriptionRefOptions<A>) => Effect.Effect<LazyRef<A, E>, never, R | Scope.Scope>;
export declare function make<A, E = never, R = never>(input: Effect.Effect<A, E, R> | Stream.Stream<A, E, R>, options?: SubscriptionRefOptions<A>): Effect.Effect<LazyRef<A, E>, never, R | Scope.Scope>;
export declare function of<A, E = never>(value: A, options?: SubscriptionRefOptions<A>): Effect.Effect<LazyRef<A, E>, never, Scope.Scope>;
export declare function failCause<E, A = unknown>(cause: Cause.Cause<E>, options?: SubscriptionRefOptions<A>): Effect.Effect<LazyRef<A, E>, never, Scope.Scope>;
export declare function fail<E, A = unknown>(error: E, options?: SubscriptionRefOptions<A>): Effect.Effect<LazyRef<A, E>, never, Scope.Scope>;
export declare function get<A, E, R>(ref: LazyRef<A, E, R>): Effect.Effect<A, E, R>;
export declare const set: {
    <const A>(value: A): <E, R>(ref: LazyRef<A, E, R>) => Effect.Effect<A, never, R>;
    <A, E, R>(ref: LazyRef<A, E, R>, a: A): Effect.Effect<A, never, R>;
};
export declare function reset<A, E, R>(ref: LazyRef<A, E, R>): Effect.Effect<Option.Option<A>, E, R>;
export { reset as delete };
export declare const modify: {
    <A, const B>(f: (a: A) => readonly [B, A]): <E, R>(ref: LazyRef<A, E, R>) => Effect.Effect<B, E, R>;
    <A, E, R, const B>(ref: LazyRef<A, E, R>, f: (a: A) => readonly [B, A]): Effect.Effect<B, E, R>;
};
export declare const modifyEffect: {
    <A, const B, E2, R2>(f: (a: A) => Effect.Effect<readonly [B, A], E2, R2>): <E, R>(ref: LazyRef<A, E, R>) => Effect.Effect<B, E | E2, R | R2>;
    <A, E, R, const B, E2, R2>(ref: LazyRef<A, E, R>, f: (a: A) => Effect.Effect<readonly [B, A], E2, R2>): Effect.Effect<B, E | E2, R | R2>;
};
export declare const update: {
    <const A>(f: (a: A) => A): <E, R>(ref: LazyRef<A, E, R>) => Effect.Effect<A, E, R>;
    <const A, E, R>(ref: LazyRef<A, E, R>, f: (a: A) => A): Effect.Effect<A, E, R>;
};
export declare const getAndUpdate: {
    <A, B>(f: (a: A) => A): <E, R>(ref: LazyRef<A, E, R>) => Effect.Effect<A, E, R>;
    <A, E, R, B>(ref: LazyRef<A, E, R>, f: (a: A) => A): Effect.Effect<A, E, R>;
};
export declare const updateEffect: {
    <A, B, E2, R2>(f: (a: A) => Effect.Effect<B, E2, R2>): <E, R>(ref: LazyRef<A, E, R>) => Effect.Effect<B, E | E2, R | R2>;
    <A, E, R, B, E2, R2>(ref: LazyRef<A, E, R>, f: (a: A) => Effect.Effect<B, E2, R2>): Effect.Effect<B, E | E2, R | R2>;
};
export declare const getAndUpdateEffect: {
    <A, E2, R2>(f: (a: A) => Effect.Effect<A, E2, R2>): <E, R>(ref: LazyRef<A, E, R>) => Effect.Effect<A, E | E2, R | R2>;
    <A, E, R, E2, R2>(ref: LazyRef<A, E, R>, f: (a: A) => Effect.Effect<A, E2, R2>): Effect.Effect<A, E | E2, R | R2>;
};
export declare const getAndSet: {
    <A>(a: A): <E, R>(ref: LazyRef<A, E, R>) => Effect.Effect<A, E, R>;
    <A, E, R>(ref: LazyRef<A, E, R>, a: A): Effect.Effect<A, E, R>;
};
export declare const mapEffect: {
    <A, B, E2, R2>(f: (a: A) => Effect.Effect<B, E2, R2>, options?: Parameters<typeof Stream.flatMap>[2]): <E, R>(ref: Computed.Computed<A, E, R>) => Computed.Computed<B, E | E2, R | R2>;
    <A, E, R, B, E2, R2>(ref: Computed.Computed<A, E, R>, f: (a: A) => Effect.Effect<B, E2, R2>, options?: Parameters<typeof Stream.flatMap>[2]): Computed.Computed<B, E | E2, R | R2>;
};
export declare const map: {
    <A, B>(f: (a: A) => B): <E, R>(ref: Computed.Computed<A, E, R>) => Computed.Computed<B, E, R>;
    <A, E, R, B>(ref: Computed.Computed<A, E, R>, f: (a: A) => B): Computed.Computed<B, E, R>;
};
export declare const proxy: <A extends Readonly<Record<PropertyKey, any>> | ReadonlyArray<any>, E, R>(source: Computed.Computed<A, E, R>) => { readonly [K in keyof A]: Computed.Computed<A[K], E, R>; };
export declare function isSubscriptionRef<A = unknown, E = unknown, R = unknown>(value: unknown): value is LazyRef<A, E, R>;
export declare function tuple<const Refs extends ReadonlyArray<LazyRef<any, any, any>>>(...refs: Refs): LazyRef<{
    readonly [K in keyof Refs]: LazyRef.Success<Refs[K]>;
}, LazyRef.Error<Refs[keyof Refs]>, LazyRef.Context<Refs[keyof Refs]>>;
export declare function tuple<const Refs extends ReadonlyArray<LazyRef<any, any, any> | Computed.Computed<any, any, any>>>(...refs: Refs): Computed.Computed<{
    readonly [K in keyof Refs]: Computed.Computed.Success<Refs[K]>;
}, LazyRef.Error<Refs[keyof Refs]>, LazyRef.Context<Refs[keyof Refs]>>;
export declare function struct<const Refs extends Readonly<Record<string, LazyRef<any, any, any>>>>(refs: Refs): LazyRef<{
    readonly [K in keyof Refs]: LazyRef.Success<Refs[K]>;
}, LazyRef.Error<Refs[keyof Refs]>, LazyRef.Context<Refs[keyof Refs]>>;
export declare function struct<const Refs extends Readonly<Record<string, LazyRef<any, any, any> | Computed.Computed<any, any, any>>>>(refs: Refs): Computed.Computed<{
    readonly [K in keyof Refs]: Computed.Computed.Success<Refs[K]>;
}, LazyRef.Error<Refs[keyof Refs]>, LazyRef.Context<Refs[keyof Refs]>>;
export interface Tagged<Self, Id extends string, A, E = never> extends LazyRef<A, E, Self> {
    readonly id: Id;
    readonly tag: Context.Tag<Self, LazyRef<A, E>>;
    readonly of: (value: A) => Layer.Layer<Self>;
    readonly make: <R>(fxOrEffect: Stream.Stream<A, E, R | Scope.Scope> | Effect.Effect<A, E, R | Scope.Scope>, options?: SubscriptionRefOptions<A> & {
        readonly drop?: number;
        readonly take?: number;
    }) => Layer.Layer<Self, never, R>;
    readonly layer: <E2, R2>(make: Effect.Effect<LazyRef<A, E>, E2, R2 | Scope.Scope>) => Layer.Layer<Self, E2, R2>;
}
export interface TaggedClass<Self, Id extends string, A, E> extends Tagged<Self, Id, A, E> {
    new (): Tagged<Self, Id, A, E>;
}
export declare function Tag<Id extends string>(id: Id): <Self, A, E = never>() => TaggedClass<Self, Id, A, E>;
export declare function fromTag<Id, S, A, E, R>(tag: Context.Tag<Id, S>, f: (s: S) => LazyRef<A, E, R>): LazyRef<A, E, Id | R>;
export declare const when: {
    <B, C>(matchers: {
        onFalse: () => B;
        onTrue: () => C;
    }): <E, R>(ref: Computed.Computed<boolean, E, R>) => Computed.Computed<B | C, E, R>;
    <E, R, B, C>(ref: Computed.Computed<boolean, E, R>, matchers: {
        onFalse: () => B;
        onTrue: () => C;
    }): Computed.Computed<B | C, E, R>;
};
export declare const provide: {
    <S, R2 = never>(contextOrLayerOrRuntime: Context.Context<S> | Layer.Layer<S, never, R2> | Runtime.Runtime<S>): {
        <A, E, R>(ref: LazyRef<A, E, R>): LazyRef<A, E, Exclude<R, S> | R2>;
        <A, E, R>(ref: Computed.Computed<A, E, R>): Computed.Computed<A, E, Exclude<R, S> | R2>;
    };
    <A, E, R, S, R2 = never>(ref: LazyRef<A, E, R>, contextOrLayerOrRuntime: Context.Context<S> | Layer.Layer<S, never, R2> | Runtime.Runtime<S>): LazyRef<A, E, Exclude<R, S> | R2>;
    <A, E, R, S, R2 = never>(ref: Computed.Computed<A, E, R>, contextOrLayerOrRuntime: Context.Context<S> | Layer.Layer<S, never, R2> | Runtime.Runtime<S>): LazyRef<A, E, Exclude<R, S> | R2>;
};
export declare const provideService: {
    <Id, S>(tag: Context.Tag<Id, S>, services: S): {
        <A, E, R>(ref: LazyRef<A, E, R>): LazyRef<A, E, Exclude<R, Id>>;
        <A, E, R>(ref: Computed.Computed<A, E, R>): Computed.Computed<A, E, Exclude<R, Id>>;
    };
    <A, E, R, Id, S>(ref: LazyRef<A, E, R>, tag: Context.Tag<Id, S>, services: S): LazyRef<A, E, Exclude<R, Id>>;
    <A, E, R, Id, S>(ref: Computed.Computed<A, E, R>, tag: Context.Tag<Id, S>, services: S): Computed.Computed<A, E, Exclude<R, Id>>;
};
export declare const provideServiceEffect: {
    <Id, S, R2>(tag: Context.Tag<Id, S>, service: Effect.Effect<S, never, R2>): {
        <A, E, R>(ref: LazyRef<A, E, R>): LazyRef<A, E, Exclude<R, Id> | R2>;
        <A, E, R>(ref: Computed.Computed<A, E, R>): Computed.Computed<A, E, Exclude<R, Id> | R2>;
    };
    <A, E, R, Id, S, R2>(ref: LazyRef<A, E, R>, tag: Context.Tag<Id, S>, service: Effect.Effect<S, never, R2>): LazyRef<A, E, Exclude<R, Id> | R2>;
    <A, E, R, Id, S, R2>(ref: Computed.Computed<A, E, R>, tag: Context.Tag<Id, S>, service: Effect.Effect<S, never, R2>): Computed.Computed<A, E, Exclude<R, Id> | R2>;
};
export declare const withSpan: {
    (name: string, options?: {
        readonly attributes?: Record<string, unknown>;
        readonly links?: ReadonlyArray<Tracer.SpanLink>;
        readonly parent?: Tracer.ParentSpan;
        readonly root?: boolean;
        readonly context?: Context.Context<never>;
    }): <A, E, R>(self: LazyRef<A, E, R>) => LazyRef<A, E, R>;
    <A, E, R>(self: LazyRef<A, E, R>, name: string, options?: {
        readonly attributes?: Record<string, unknown>;
        readonly links?: ReadonlyArray<Tracer.SpanLink>;
        readonly parent?: Tracer.ParentSpan;
        readonly root?: boolean;
        readonly context?: Context.Context<never>;
    }): LazyRef<A, E, R>;
};
export declare const drop: {
    (n: number): {
        <A, E, R>(ref: LazyRef<A, E, R>): LazyRef<A, E, R>;
        <A, E, R>(ref: Computed.Computed<A, E, R>): Computed.Computed<A, E, R>;
    };
    <A, E, R>(ref: LazyRef<A, E, R>, n: number): LazyRef<A, E, R>;
    <A, E, R>(ref: Computed.Computed<A, E, R>, n: number): Computed.Computed<A, E, R>;
};
export declare const take: {
    (n: number): {
        <A, E, R>(ref: LazyRef<A, E, R>): LazyRef<A, E, R>;
        <A, E, R>(ref: Computed.Computed<A, E, R>): Computed.Computed<A, E, R>;
    };
    <A, E, R>(ref: LazyRef<A, E, R>, n: number): LazyRef<A, E, R>;
    <A, E, R>(ref: Computed.Computed<A, E, R>, n: number): Computed.Computed<A, E, R>;
};
