import { Stream } from 'effect';
import * as Effect from 'effect/Effect';
import * as FiberRefsPatch from 'effect/FiberRefsPatch';
import * as Layer from 'effect/Layer';
import * as Runtime from 'effect/Runtime';
import * as RuntimeFlags from 'effect/RuntimeFlags';
export const ProvideContext = (i0) => ({
    _tag: 'ProvideContext',
    i0,
});
export const ProvideLayer = (i0) => ({
    _tag: 'ProvideLayer',
    i0,
});
export const ProvideService = (i0, i1) => ({
    _tag: 'ProvideService',
    i0,
    i1,
});
export const ProvideServiceEffect = (i0, i1) => ({
    _tag: 'ProvideServiceEffect',
    i0,
    i1,
});
export const ProvideRuntime = (i0) => ({
    _tag: 'ProvideRuntime',
    i0,
});
export function matchProvide(self, matchers) {
    return matchers[self._tag](self.i0, self.i1);
}
export function merge(self, that) {
    return ProvideLayer(Layer.provideMerge(toLayer(self), toLayer(that)));
}
export function buildWithScope(provide, scope) {
    return Layer.buildWithScope(toLayer(provide), scope);
}
export function toLayer(provide) {
    switch (provide._tag) {
        case 'ProvideContext':
            return Layer.succeedContext(provide.i0);
        case 'ProvideLayer':
            return provide.i0;
        case 'ProvideRuntime':
            return runtimeToLayer(provide.i0);
        case 'ProvideService':
            return Layer.succeed(provide.i0, provide.i1);
        case 'ProvideServiceEffect':
            return Layer.effect(provide.i0, provide.i1);
    }
}
export function provideToEffect(effect, provide) {
    return Effect.provide(effect, toLayer(provide));
}
export function provideToStream(stream, provide) {
    return Stream.provideSomeLayer(stream, toLayer(provide));
}
export function runtimeToLayer(runtime) {
    // Calculate patch
    const patchRefs = FiberRefsPatch.diff(Runtime.defaultRuntime.fiberRefs, runtime.fiberRefs);
    const patchFlags = RuntimeFlags.diff(Runtime.defaultRuntime.runtimeFlags, runtime.runtimeFlags);
    return Layer.scopedContext(Effect.Do.pipe(
    // Get Current Refs + Flags
    Effect.bind('oldRefs', () => Effect.getFiberRefs), Effect.bind('oldFlags', () => Effect.getRuntimeFlags), 
    // Patch Refs + Flags
    Effect.tap(() => Effect.patchFiberRefs(patchRefs)), Effect.tap(() => Effect.patchRuntimeFlags(patchFlags)), 
    // Get the new Refs + Flags
    Effect.bind('newRefs', () => Effect.getFiberRefs), Effect.bind('newFlags', () => Effect.getRuntimeFlags), 
    // Calculate rollback patch
    Effect.let('rollbackRefs', ({ newRefs, oldRefs }) => FiberRefsPatch.diff(newRefs, oldRefs)), Effect.let('rollbackFlags', ({ newFlags, oldFlags }) => RuntimeFlags.diff(newFlags, oldFlags)), 
    // Apply the rollbacks when the current scope is closed
    Effect.tap(({ rollbackFlags, rollbackRefs }) => Effect.addFinalizer(() => Effect.zipRight(Effect.patchFiberRefs(rollbackRefs), Effect.patchRuntimeFlags(rollbackFlags)))), 
    // Provide the runtime's context
    Effect.map(() => runtime.context)));
}
//# sourceMappingURL=Provide.js.map