import { Effect, Readable, Record, Stream, Subscribable } from 'effect';
import { EffectBase, MulticastEffect } from './internal.js';
export function make(get, changes, version) {
    return new VersionedImpl(get, changes, version);
}
/**
 * @internal
 */
export class VersionedImpl extends EffectBase {
    changes;
    version;
    [Subscribable.TypeId] = Subscribable.TypeId;
    [Readable.TypeId] = Readable.TypeId;
    get;
    constructor(get, changes, version) {
        super();
        this.changes = changes;
        this.version = version;
        this.get = new MulticastEffect(get);
    }
    toEffect() {
        return this.get;
    }
}
export function tuple(...versioneds) {
    return new TupleImpl(versioneds);
}
class TupleImpl extends EffectBase {
    versioneds;
    [Subscribable.TypeId] = Subscribable.TypeId;
    [Readable.TypeId] = Readable.TypeId;
    version;
    changes;
    constructor(versioneds) {
        super();
        this.versioneds = versioneds;
        this.version = Effect.all(versioneds, { concurrency: 'unbounded' }).pipe((_) => _, Effect.map((versions) => versions.reduce((acc, version) => acc + version, 0)));
        this.changes = Stream.zipLatestAll(...versioneds.map((v) => v.changes));
    }
    get get() {
        return this;
    }
    toEffect() {
        return Effect.all(this.versioneds, {
            concurrency: 'unbounded',
        });
    }
}
export function struct(refs) {
    return new StructImpl(refs);
}
class StructImpl extends EffectBase {
    versioneds;
    [Subscribable.TypeId] = Subscribable.TypeId;
    [Readable.TypeId] = Readable.TypeId;
    version;
    changes;
    constructor(versioneds) {
        super();
        this.versioneds = versioneds;
        this.version = Effect.all(Object.values(versioneds), { concurrency: 'unbounded' }).pipe((_) => _, Effect.map((versions) => versions.reduce((acc, version) => acc + version, 0)));
        this.changes = Stream.map(Stream.zipLatestAll(...Object.entries(versioneds).map(([key, { changes }]) => changes.pipe(Stream.map((value) => [key, value])))), Record.fromEntries);
    }
    get get() {
        return this;
    }
    toEffect() {
        return Effect.all(this.versioneds, { concurrency: 'unbounded' });
    }
}
//# sourceMappingURL=Versioned.js.map