import { Cause, Deferred, Effect, Effectable, Equivalence, Exit, Fiber, MutableRef, Option, PubSub, Queue, Scope, Stream, type FiberId, type Runtime, type Types } from 'effect';
import type { GetSetDelete, SubscriptionRefOptions } from './LazyRef.js';
/**
 * @internal
 */
export declare const deepEquals: (a: unknown, b: unknown) => boolean;
export declare abstract class EffectBase<A, E, R> extends Effectable.StructuralClass<A, E, R> implements Effect.Effect<A, E, R> {
    abstract toEffect(): Effect.Effect<A, E, R>;
    private _effect;
    commit(): Effect.Effect<A, E, R>;
}
export declare class SubscriptionRefCore<A, E, R, R2> {
    readonly initial: Effect.Effect<A, E, R>;
    readonly pubsub: PubsubWithReplay<A, E>;
    readonly runtime: Runtime.Runtime<R2>;
    readonly scope: Scope.CloseableScope;
    readonly deferredRef: DeferredRef<A, E>;
    readonly semaphore: Effect.Semaphore;
    constructor(initial: Effect.Effect<A, E, R>, pubsub: PubsubWithReplay<A, E>, runtime: Runtime.Runtime<R2>, scope: Scope.CloseableScope, deferredRef: DeferredRef<A, E>, semaphore: Effect.Semaphore);
    _fiber: Fiber.Fiber<A, E> | undefined;
}
export declare function makeCore<A, E, R>(initial: Effect.Effect<A, E, R>, options?: SubscriptionRefOptions<A>): Effect.Effect<SubscriptionRefCore<A, E, R, Scope.Scope | R>, never, Scope.Scope | R>;
export declare function unsafeMakeCore<A, E, R, R2>(initial: Effect.Effect<A, E, R>, id: FiberId.FiberId, runtime: Runtime.Runtime<R2>, scope: Scope.CloseableScope, options?: SubscriptionRefOptions<A>): SubscriptionRefCore<A, E, R, R2>;
export declare function matchEffectPrimitive<A, E, R, Z>(effect: Effect.Effect<A, E, R>, matchers: {
    Success: (a: A) => Z;
    Failure: (e: Cause.Cause<E>) => Z;
    Sync: (f: () => A) => Z;
    Left: (e: E) => Z;
    Right: (a: A) => Z;
    Some: (a: A) => Z;
    None: (e: E) => Z;
    Otherwise: (effect: Effect.Effect<A, E, R>) => Z;
}): Z;
export declare const getExitEquivalence: <A, E = never>(A: Equivalence.Equivalence<A>) => Equivalence.Equivalence<Exit.Exit<A, E>>;
declare class PubsubWithReplay<A, E> implements PubSub.PubSub<Exit.Exit<A, E>> {
    readonly pubsub: PubSub.PubSub<Exit.Exit<A, E>>;
    readonly lastValue: MutableRef.MutableRef<Option.Option<Exit.Exit<A, E>>>;
    [Queue.EnqueueTypeId]: {
        _In: Types.Contravariant<Exit.Exit<A, E>>;
    };
    readonly subscriberCount: MutableRef.MutableRef<number>;
    constructor(pubsub: PubSub.PubSub<Exit.Exit<A, E>>, lastValue: MutableRef.MutableRef<Option.Option<Exit.Exit<A, E>>>);
    publish(value: Exit.Exit<A, E>): Effect.Effect<boolean>;
    publishAll(elements: Iterable<Exit.Exit<A, E>>): Effect.Effect<boolean>;
    subscribe: Effect.Effect<Queue.Dequeue<Exit.Exit<A, E>>, never, Scope.Scope>;
    offer(value: Exit.Exit<A, E>): Effect.Effect<boolean>;
    unsafeOffer(value: Exit.Exit<A, E>): boolean;
    offerAll(elements: Iterable<Exit.Exit<A, E>>): Effect.Effect<boolean>;
    capacity(): number;
    isActive(): boolean;
    get size(): Effect.Effect<number>;
    unsafeSize(): Option.Option<number>;
    get isFull(): Effect.Effect<boolean>;
    get isEmpty(): Effect.Effect<boolean>;
    get isShutdown(): Effect.Effect<boolean>;
    get shutdown(): Effect.Effect<void>;
    get awaitShutdown(): Effect.Effect<void>;
    pipe(): unknown;
}
export declare function dequeuePrepend<A>(dequeue: Queue.Dequeue<A>, previous: A): Queue.Dequeue<A>;
export declare function pubsubWithReplay<A, E>(): PubsubWithReplay<A, E>;
export declare function streamExit<A, E, R>(stream: Stream.Stream<A, E, R>): Stream.Stream<Exit.Exit<A, E>, never, R>;
export declare class DeferredRef<A, E> extends EffectBase<A, E, never> {
    private id;
    private eq;
    readonly current: MutableRef.MutableRef<Option.Option<Exit.Exit<A, E>>>;
    version: number;
    deferred: Deferred.Deferred<A, E>;
    constructor(id: FiberId.FiberId, eq: Equivalence.Equivalence<Exit.Exit<A, E>>, current: MutableRef.MutableRef<Option.Option<Exit.Exit<A, E>>>);
    toEffect(): Effect.Effect<A, E, never>;
    done(exit: Exit.Exit<A, E>): boolean;
    reset(): void;
}
export declare function makeDeferredRef<A, E>(eq: Equivalence.Equivalence<Exit.Exit<A, E>>): Effect.Effect<DeferredRef<A, E>, never, never>;
export declare function unsafeMakeDeferredRef<A, E>(id: FiberId.FiberId, eq: Equivalence.Equivalence<Exit.Exit<A, E>>, current: MutableRef.MutableRef<Option.Option<Exit.Exit<A, E>>>): DeferredRef<A, E>;
export declare class MulticastEffect<A, E, R> extends EffectBase<A, E, R> {
    readonly effect: Effect.Effect<A, E, R>;
    private _fiber;
    private _refCount;
    constructor(effect: Effect.Effect<A, E, R>);
    toEffect(): Effect.Effect<A, E, R>;
    interrupt: Effect.Effect<void, never, never>;
}
export declare function getSetDelete<A, E, R, R2>(core: SubscriptionRefCore<A, E, R, R2>): GetSetDelete<A, E, Exclude<R, R2>>;
export declare function getOrInitializeCore<A, E, R, R2>(core: SubscriptionRefCore<A, E, R, R2>, lockInitialize: boolean): Effect.Effect<A, E, Exclude<R, R2>>;
export declare function interruptCore<A, E, R, R2>(core: SubscriptionRefCore<A, E, R, R2>): Effect.Effect<void, never, R>;
export declare function interruptCoreAndWait<A, E, R, R2>(core: SubscriptionRefCore<A, E, R, R2>): Effect.Effect<void, never, R>;
export declare function sendEvent<A, E, R, R2>(core: SubscriptionRefCore<A, E, R, R2>, exit: Exit.Exit<A, E>): Effect.Effect<void>;
export {};
